      subroutine radtran

c  monte carlo radiative transfer for disk and envelope surrounding star.

c  history
c  000908 (baw) complete rewrite, to use 3-D spherical grid (subroutines
c  supplied by M. wolff).  

c     *************************************************************

      implicit none
      include 'stokes.txt'
      include 'ds.txt'
      include 'opacin.txt'
      include 'taunum.txt'
      include 'random.txt'
      include 'grid.txt'
      
      real*8 xmax,fractx,thetb,coslp,sinlp,cosnorm,sipnew,tsum,fractxh
     1     ,eps

      integer ii(3),iphot,k,gridflag,icount,iscat,it,ip,ir,i
     $,     count2,ips

      icount=0
      eps=1.d-9

c     set up density grid
      call gridset
      
c     The flux is summed to image(ix,iy,it)
c     polarization to imagei(ixp,iyp,it),imageq...,imageu...

      xmax=rmaxi

      nscat=0.
      tot=0.
      count2=0

c     x,y arrays
      fractx=2.d0*rmaxi/dfloat(nx)
      fractxh=2.d0*rmaxi/dfloat(nxhst)
c     print*,'fractxh ',fractxh

      if (rmine/rmin.gt.100) then
         ips=1
         print*,'assuming point source emission'
      else
         ips=0
      endif

      call flush(6)
      
c     ********  do loop over photons  *******
      
      do iphot=1,np
         
         if(mod(iphot,iwrite).eq.0) then
            write(6,*) iphot
            call flush(6)
         end if
c     initialize stokes parameters, incident angles
         if (ips.eq.0) then
            call initp
         else
            call initp_ps
         endif
c     zstar
         zp=cosb*rmin*(1.d0+eps)
c     rstar=xp if all photons start off on x-axis
         rp=sinb*rmin*(1.d0+eps)
         coslp=dcos(lp)
         sinlp=dsin(lp)
         xp=rp*coslp
         yp=rp*sinlp
         rsq=zp**2+rp**2
         rtot=dsqrt(rsq)
         ux=sint*cosp
         uy=sint*sinp
         uz=cost
         iflag=0
         ii(1)=1    !index of radial grid (stellar surface is 1)
         if (ntg.gt.1) then
            thetb=acos(cosb)
            call locate(thetarr,ntg,thetb,it)
            ii(2)=it
         else
            ii(2)=1
            it=1
         endif
         if (npg.gt.1) then
            call locate(phiarr,npg,lp,ip)
            ii(3)=ip
         else
            ii(3)=1
         endif
c     print*,'lp, ip ',lp,ip
c     print*,'thetb, it',thetb,it
c     ii is position in radial grid at star.
c     print*,'xp,yp,zp ',xp,yp,zp
c     print*,'ux,uy,uz ',ux,uy,uz

c     first, peel off direct flux
c     weight photon intensity by angle between normal and 
c     photon direction, since emitted from a surface.
         if (ipeel.eq.1) then
            cosnorm=cosb*coste+(sinb*sinte*(cospe*coslp+sinpe*sinlp))
            if (limb.eq.0) then
c     intensity constant, energy/Sr proportional to mu
               sipnew=4.*sip*cosnorm !normalization = 2
            else
c     intensity goes as (1+mu).  energy/Sr has another factor
c     of mu.
               sipnew=12.d0/5.d0*sip*(cosnorm+cosnorm*cosnorm)
            endif
c     sipnew=sip	 
            if (cosnorm.gt.0.d0) then     
               call peeloff(xp,yp,zp,sipnew,sqp,sup,svp
     1              ,cost,sint,cosp,sinp,phi
     1              ,pi,r2p,hit,htot,rsq,rtot
     1              ,peak,tsum,fractxh,ii,idust,iflag,iphot,eps)
            endif
         endif

c     now calculate scattered flux
         call propagate(iphot,xmax,fractx,fractxh,ii,count2,eps)

 400  end do
      
      write(6,*) 'sini,cosi'
      write(6,*) (sini(i),i=1,nmu),(u(i),i=1,nmu)
      write(6,*) 'fractx,xmax',fractx,xmax
c     ***** end of loop over photons  ******
      
      write(6,*) 'fraction of phots scattered outside image'
      write(6,*) float(icount)/float(np)
      print*, 'number of shouldnt be here photons ', count2
      return
      end
      
      
c     *********************************************************
      

